"""
Adapted from FEniCS tutorial program for nonlinearly elastic material 

The model is used to simulate an elastic 2d cornea clamped at its top and bottom end (limbus) and deformed under IOP coming from the left (posterior end). We also model keratoconus as damage to lamellar segments and diagonal crosslinks which varies quartically (as default) from limbus to apex (max damage - meaning lowest stiffness - at apex).
"""

from __future__ import print_function
from fenics import *
from dolfin import *
import numpy
import csv

# Model parameters

T = 0.62 # mm - corneal thickness
D_A = 11.46 # mm - in-plane anterior diameter 
R_A =  7.8 # mm - anterior radius
R_P = R_A-T # mm - posterior radius
PhiStar = numpy.arcsin(D_A/(2*R_A)) # dimless - half of aperture angle
N = 64 # dimless - finite element mesh resolution in the radial direction; default is 64
gamma = 20 # dimless
M = N*gamma # dimless
kappaDimensional = 2*PhiStar/(gamma*T) # mm^{-1} - this is dimensional counterpart of kappa from the manuscript
alpha1 = 638 # kPa - material parameter from "The role of 3-d collagen organization in stromal elasticity: a model based on x-ray diffraction data and second harmonic-genereated images."
E = 4*alpha1 # kPa - linearization of model from "The role of 3-d collagen organization in stromal elasticity: a model based on x-ray diffraction data and second harmonic-genereated images." 
K1 = 3*E*gamma*T/(R_A*2*PhiStar) # kPa*mm - stiffness of family 1 (using dimensional out-of-plane depth = 1mm)
K2 = 0.001*K1 # kPa*mm - stiffness of family 2; same ratio as in "Modeling the degeneration of the collagen architecture in a microstructural model of the human cornea"
K3 = 0.07*K1 # kPa*mm - stiffness of family 3; same ratio as in "Modeling the degeneration of the collagen architecture in a microstructural model of the human cornea"

# define rectangular mesh and transform onto cylindrical geometry

mesh = RectangleMesh(Point(R_P, -0.5), Point(R_A, 0.5), N, M, "crossed")

x = mesh.coordinates()[:,0]
y = mesh.coordinates()[:,1]
coordinates = mesh.coordinates()

def cylinder(r, s):
   return [r*numpy.cos(2*PhiStar*s),r*numpy.sin(2*PhiStar*s)]

x_hat, y_hat = cylinder(x, y)
xy_hat_coor = numpy.array([x_hat, y_hat]).transpose()
mesh.coordinates()[:] = xy_hat_coor

# Create mesh and define function space

V = VectorFunctionSpace(mesh, 'P', 1)
VScalar = FunctionSpace(mesh, 'P', 1)

# Define boundary condition

tol = 1E-14 # tolerance for boundary (part) identification

boundary_parts = \
MeshFunction("size_t", mesh, mesh.topology().dim()-1)

boundary_parts.set_all(1)

class pinned_boundary(SubDomain):
   def inside(self,x,on_boundary):
      return on_boundary and abs(abs(numpy.arctan(x[1]/x[0]))-PhiStar)<tol

pinned_boundary().mark(boundary_parts, 2)

TolCirc = 1E-3 # tolerance for boundary (part) identification for posterior and anterior boundaries

class PosteriorBoundary(SubDomain):
	def inside(self, x, on_boundary):
		return on_boundary and x[0]*x[0]+x[1]*x[1]<R_P*R_P+TolCirc
PosteriorBoundary().mark(boundary_parts, 3)

class AnteriorBoundary(SubDomain):
	def inside(self, x, on_boundary):
		return on_boundary and x[0]*x[0]+x[1]*x[1]>R_A*R_A-TolCirc
AnteriorBoundary().mark(boundary_parts, 4)

bc = DirichletBC(V, Constant((0, 0)), pinned_boundary())

dss = ds(domain=mesh, subdomain_data= boundary_parts)

# Define all functions and expressions needed

PolarR = Expression("sqrt(x[0]*x[0]+x[1]*x[1])",degree=1) # this is dimensional but so is kappaDimensional

UnitE1 = Expression(("1","0"),degree=1)

UnitE2 = Expression(("0","1"),degree=1)

UnitER = Expression(("x[0]/PolarR","x[1]/PolarR"),PolarR=PolarR,degree=1)

UnitEPhi = Expression(("-x[1]/PolarR","x[0]/PolarR"),PolarR=PolarR,degree=1)

DiagonalLength = Expression("sqrt(1+kappaDimensional*kappaDimensional*PolarR*PolarR)",kappaDimensional=kappaDimensional,PolarR=PolarR,degree=1)

UnitEDiagPlus = Expression(("(x[0]-kappaDimensional*PolarR*x[1])/(DiagonalLength*PolarR)","(x[1]+kappaDimensional*PolarR*x[0])/(DiagonalLength*PolarR)"),kappaDimensional=kappaDimensional,PolarR=PolarR,DiagonalLength=DiagonalLength,degree=1)

UnitEDiagMinus = Expression(("(x[0]+kappaDimensional*PolarR*x[1])/(DiagonalLength*PolarR)","(x[1]-kappaDimensional*PolarR*x[0])/(DiagonalLength*PolarR)"),kappaDimensional=kappaDimensional,PolarR=PolarR,DiagonalLength=DiagonalLength,degree=1)

# define microscale constitutive laws (linear springs)

def E1(y):
    return K1/2*(y-1)**2

def E2(y):
    return K2/2*(y-1)**2
    
def E3(y):
    return K3/2*(y-1)**2

# Define functions for numerical solution
du = TrialFunction(V)            # Incremental displacement
v  = TestFunction(V)             # Test function
u  = Function(V)                 # Displacement from previous iteration

# Kinematics
d = u.geometric_dimension()
I = Identity(d)             # Identity tensor
F = I + grad(u)             # Deformation gradient
C = F.T*F                   # Right Cauchy-Green tensor

JDet  = det(F) # determinant of F
Finv = inv(F) # inverse of F

p = 2.0 #kPa - IOP

YMax = R_A*sin(PhiStar) # mm - maximum value of (dimensional) Y coordinate
DamageExponent=4 # dimless

for Damage in [0.0]: # can model damage by varying Damage as (e.g.): [0.0,0.80,0.90,0.95,0.99]:
  print(Damage)
  DamageProfile = Expression("1-Damage*(1-pow(x[1]/YMax,DamageExponent))",YMax = YMax,Damage=Damage,DamageExponent=DamageExponent,degree=1)
  # Strain energy density 
  psi = 1/(kappaDimensional*PolarR)*(E2(sqrt(dot(UnitER,dot(C,UnitER)))) + (kappaDimensional*PolarR)**2 *DamageProfile*E1(sqrt(dot(UnitEPhi,dot(C,UnitEPhi))))+DiagonalLength**2 *DamageProfile*(E3(sqrt(dot(UnitEDiagPlus,dot(C,UnitEDiagPlus)))) +E3(sqrt(dot(UnitEDiagMinus,dot(C,UnitEDiagMinus))))))
  # Total potential energy including term arising from posterior boundary condition (IOP)
  Pi = psi*dx - p*JDet*dot(u,dot(Finv.T,UnitER))*dss(3)
  # Compute first variation of Pi (directional derivative about u in the direction of v)
  FF = derivative(Pi, u, v)
  # Compute Jacobian of FF
  J = derivative(FF, u, du)
  # Solve variational problem
  solve(FF == 0, u, bc, J=J)
  # Save solution in VTK format
  file = File("displacementConusDamage"+str(Damage)+"Exponent"+str(DamageExponent)+".pvd");
  file << u;  

  # Deduce X and Y components of the displacement field
  uX = project(dot(UnitE1,u),VScalar)
  uY = project(dot(UnitE2,u),VScalar) 
  # Extract deformed anterior and posterior surfaces and save them in .csv files
  AnteriorCurrent = []
  PosteriorCurrent = []
  for i, x in enumerate(coordinates):
    if (x[0]*x[0]+x[1]*x[1]>R_A*R_A-TolCirc):
      AnteriorCurrent.append([x[0]+uX(x),x[1]+uY(x)])
    elif (x[0]*x[0]+x[1]*x[1]<R_P*R_P+TolCirc):
      PosteriorCurrent.append([x[0]+uX(x),x[1]+uY(x)])

  with open("PosteriorCurrentConusDamage"+str(Damage)+"Exponent"+str(DamageExponent)+".csv", "w", newline="") as f:
    writer = csv.writer(f)
    writer.writerows(PosteriorCurrent)
   
  with open("AnteriorCurrentConusDamage"+str(Damage)+"Exponent"+str(DamageExponent)+".csv", "w", newline="") as f:
    writer = csv.writer(f)
    writer.writerows(AnteriorCurrent)