clear all
clc
close all

R_A = 7.8; % mm  - anterior radius
T = 0.62; % mm - corneal thickness (at the limbus)
D_A = 11.46; % mm - in-plane anterior diameter
phiStar = asin(D_A/(2*R_A)) % dimless - half of aperture angle

alpha1 = 638; %kPa - material parameter from "The role of 3-d collagen organization in stromal elasticity: a model based on x-ray diffraction data and second harmonic-generated images."
E = 4*alpha1; %kPa - linearization of model from "The role of 3-d collagen organization in stromal elasticity: a model based on x-ray diffraction data and second harmonic-generated images."

gamma=20; % dimless - modulates the aspect ratio

kColl=3*E*gamma*T/(2*R_A*phiStar) %kPa*mm - stiffness of family 1 (using dimensional out-of-plane depth=1 mm)
kCLOrth = 0.001*kColl; %kPa*mm - stiffness of family 2; same ratio as in "Modeling the degeneration of the collagen architecture in a microstructural model of the human cornea"
kCLDiag =0.07*kColl; %kPa*mm - stiffness of family 3; same ratio as in "Modeling the degeneration of the collagen architecture in a microstructural model of the human cornea"

for N=2.^(1:2) % number of lamellae through the corneal thickness; vary as N=2,4,8,16,32 and 64 by setting "1:6"

M=20*N
exitflag = 1; % exitflag determining whether solver found the solution or not

% Following lines create the unloaded (circular configuration) which is also the initial guess 

RVector = linspace(R_A - T , R_A, N+1);
phiVector = linspace(-phiStar,phiStar,M+1);

for ii=1:(N+1)
    for jj=1:(M+1)    
                x0(jj,ii,1)= RVector(ii)*cos(phiVector(jj));
                x0(jj,ii,2)= RVector(ii)*sin(phiVector(jj));
                X(jj,ii,1)= RVector(ii)*cos(phiVector(jj));
                X(jj,ii,2)= RVector(ii)*sin(phiVector(jj));
    end
end

pInit=2.0 % kPa (equivalent to approximately 15 mmHg) - IOP
p=pInit;

while (exitflag>0 & p<=2.0001) % one can gradually increase IOP in a loop
    [exitflag,x0]=SolveAtGivenIOP(M,N,kCLDiag,kCLOrth,kColl,p,R_A,D_A,T,phiStar,x0,X,RVector,phiVector);
    p=p+0.5
end

end

    
function [exitflag,x0]=SolveAtGivenIOP(M,N,kCLDiag,kCLOrth,kColl,p,R_A,D_A,T,phiStar,x0,X,RVector,phiVector)

fun = @root2d;
options = optimoptions('fsolve'); % to change error tolerances:  ,'StepTolerance',1e-6,'FunctionTolerance',1e-6); 

[x,fval,exitflag,output] = fsolve(fun,x0,options); % solve the discrete problem
exitflag % check the exitflag - did fsolve find the solution?



% Export the solution into .csv files
if (exitflag>0)
dlmwrite(strcat('XComponentsIOP',sprintf('%.1f',p),'_HealthyN',int2str(N),'.csv'),x(:,:,1), 'delimiter', ',', 'precision', 17)
dlmwrite(strcat('YComponentsIOP',sprintf('%.1f',p),'_HealthyN',int2str(N),'.csv'),x(:,:,2), 'delimiter', ',', 'precision', 17)
end

% Rewrite the initial guess
for ii=1:(N+1)
    for jj=1:(M+1)
           x0(jj,ii,1)= x(jj,ii,1);
           x0(jj,ii,2)= x(jj,ii,2);
    end
end


% Function specifying the full discrete problem, including the boundary
% conditions
function F = root2d(x)

for i=1:(N+1)
    for j=1:(M+1)
        if (j==1 | j==M+1)   %limbus;    
            F(2*((N+1)*(j-1)+i)-1)= x(j,i,1)-RVector(i)*cos(phiVector(j));
            F(2*((N+1)*(j-1)+i))= x(j,i,2)- RVector(i)*sin(phiVector(j));
        else  % internal nodes but also anterior and posterior
            
                   [SignedNormBottom,FBottomX,FBottomY]= LamellarSegmentForce(x(j,i,1),x(j,i,2),x(j-1,i,1),x(j-1,i,2),sqrt((X(j-1,i,1)-X(j,i,1))^2+(X(j-1,i,2)-X(j,i,2))^2));                
                   [SignedNormTop,FTopX,FTopY]= LamellarSegmentForce(x(j,i,1),x(j,i,2),x(j+1,i,1),x(j+1,i,2),sqrt((X(j+1,i,1)-X(j,i,1))^2+(X(j+1,i,2)-X(j,i,2))^2));
                   
                   if i== (N+1) % anterior
                     FRightX=0;
                     FRightY=0;
                     [SignedNormLeft,FLeftX,FLeftY]= RadialCrosslinkForce(x(j,i,1),x(j,i,2),x(j,i-1,1),x(j,i-1,2),sqrt((X(j,i-1,1)-X(j,i,1))^2+(X(j,i-1,2)-X(j,i,2))^2));                   
                     [SignedNormBottomLeft,FBottomLeftX,FBottomLeftY]= DiagonalCrosslinkForce(x(j,i,1),x(j,i,2),x(j-1,i-1,1),x(j-1,i-1,2),sqrt((X(j-1,i-1,1)-X(j,i,1))^2+(X(j-1,i-1,2)-X(j,i,2))^2));                
                     [SignedNormTopLeft,FTopLeftX,FTopLeftY]= DiagonalCrosslinkForce(x(j,i,1),x(j,i,2),x(j+1,i-1,1),x(j+1,i-1,2),sqrt((X(j+1,i-1,1)-X(j,i,1))^2+(X(j+1,i-1,2)-X(j,i,2))^2));
                     FBottomRightX=0;
                     FBottomRightY=0;
                     FTopRightX=0;
                     FTopRightY=0;
                    elseif i==1   % posterior - IOP condition (using dimensional out-of-plane depth=1 mm)
                    [SignedNormRight,FRightX,FRightY]= RadialCrosslinkForce(x(j,i,1),x(j,i,2),x(j,i+1,1),x(j,i+1,2),sqrt((X(j,i+1,1)-X(j,i,1))^2+(X(j,i+1,2)-X(j,i,2))^2));
                    FLeftX=0.5*p*(x(j+1,1,2)-x(j-1,1,2));
                    FLeftY=0.5*p*(x(j-1,1,1)-x(j+1,1,1));
                    [SignedNormBottomRight,FBottomRightX,FBottomRightY]= DiagonalCrosslinkForce(x(j,i,1),x(j,i,2),x(j-1,i+1,1),x(j-1,i+1,2),sqrt((X(j-1,i+1,1)-X(j,i,1))^2+(X(j-1,i+1,2)-X(j,i,2))^2));                
                    [SignedNormTopRight,FTopRightX,FTopRightY]= DiagonalCrosslinkForce(x(j,i,1),x(j,i,2),x(j+1,i+1,1),x(j+1,i+1,2),sqrt((X(j+1,i+1,1)-X(j,i,1))^2+(X(j+1,i+1,2)-X(j,i,2))^2));
                     FBottomLeftX=0;
                     FBottomLeftY=0;
                     FTopLeftX=0;
                     FTopLeftY=0;
                   else % internal nodes
                     [SignedNormLeft,FLeftX,FLeftY]= RadialCrosslinkForce(x(j,i,1),x(j,i,2),x(j,i-1,1),x(j,i-1,2),sqrt((X(j,i-1,1)-X(j,i,1))^2+(X(j,i-1,2)-X(j,i,2))^2));
                     [SignedNormRight,FRightX,FRightY]= RadialCrosslinkForce(x(j,i,1),x(j,i,2),x(j,i+1,1),x(j,i+1,2),sqrt((X(j,i+1,1)-X(j,i,1))^2+(X(j,i+1,2)-X(j,i,2))^2));
                     [SignedNormBottomLeft,FBottomLeftX,FBottomLeftY]= DiagonalCrosslinkForce(x(j,i,1),x(j,i,2),x(j-1,i-1,1),x(j-1,i-1,2),sqrt((X(j-1,i-1,1)-X(j,i,1))^2+(X(j-1,i-1,2)-X(j,i,2))^2));                
                     [SignedNormTopLeft,FTopLeftX,FTopLeftY]= DiagonalCrosslinkForce(x(j,i,1),x(j,i,2),x(j+1,i-1,1),x(j+1,i-1,2),sqrt((X(j+1,i-1,1)-X(j,i,1))^2+(X(j+1,i-1,2)-X(j,i,2))^2));
                     [SignedNormBottomRight,FBottomRightX,FBottomRightY]= DiagonalCrosslinkForce(x(j,i,1),x(j,i,2),x(j-1,i+1,1),x(j-1,i+1,2),sqrt((X(j-1,i+1,1)-X(j,i,1))^2+(X(j-1,i+1,2)-X(j,i,2))^2));                
                     [SignedNormTopRight,FTopRightX,FTopRightY]= DiagonalCrosslinkForce(x(j,i,1),x(j,i,2),x(j+1,i+1,1),x(j+1,i+1,2),sqrt((X(j+1,i+1,1)-X(j,i,1))^2+(X(j+1,i+1,2)-X(j,i,2))^2));
                    end
                    
        F(2*((N+1)*(j-1)+i)-1) = (FRightX+FLeftX+FTopX+FBottomX+FTopRightX+FTopLeftX+FBottomRightX+FBottomLeftX); % Total force (X component)
        F(2*((N+1)*(j-1)+i)) = (FRightY+FLeftY+FTopY+FBottomY+FTopRightY+FTopLeftY+FBottomRightY+FBottomLeftY); % Total force (Y component)     
        
        end
    end
end
end

function [SignedNorm,Fx,Fy]= LamellarSegmentForce(x1,y1,x2,y2,deltaRest)  % Calculates force that lamellar segment experiences if its two end points are at (x1,y1) and (x2,y2)

    SignedNorm =  kColl*(sqrt((x2-x1)^2+(y2-y1)^2)-deltaRest);
    Fx = SignedNorm*(x2-x1)/sqrt((x2-x1)^2+(y2-y1)^2);
    Fy = SignedNorm*(y2-y1)/sqrt((x2-x1)^2+(y2-y1)^2);
    
end

function [SignedNorm,Fx,Fy]= RadialCrosslinkForce(x1,y1,x2,y2,deltaRest)  % Calculates force that radial crosslink experiences if its two end points are at (x1,y1) and (x2,y2)

    SignedNorm = kCLOrth*(sqrt((x2-x1)^2+(y2-y1)^2)- deltaRest);
    Fx = SignedNorm*(x2-x1)/sqrt((x2-x1)^2+(y2-y1)^2);
    Fy = SignedNorm*(y2-y1)/sqrt((x2-x1)^2+(y2-y1)^2);
    
end

function [SignedNorm,Fx,Fy]= DiagonalCrosslinkForce(x1,y1,x2,y2,deltaRest)  % Calculates force that diagonal crosslink experiences if its two end points are at (x1,y1) and (x2,y2)

    SignedNorm = kCLDiag*(sqrt((x2-x1)^2+(y2-y1)^2)- deltaRest);
    Fx = SignedNorm*(x2-x1)/sqrt((x2-x1)^2+(y2-y1)^2);
    Fy = SignedNorm*(y2-y1)/sqrt((x2-x1)^2+(y2-y1)^2);
    
end

end